#include "CMidiMsg.h"

CMidiMsg::CMidiMsg(void)
{
	clearMidiMsg();
}

CMidiMsg::~CMidiMsg(void)
{
}

void CMidiMsg::clearMidiMsg()
{
	// o[ϐ
	cur=0;
	num=0;
	memset(buf, 0, sizeof(MidiMessage)*MIDIMSG_MAXNUM);
}

int CMidiMsg::addMidiMsg(VstMidiEvent *midievent)
{
	MidiMessage tmp;
	tmp.deltaFrames = midievent->deltaFrames;
	tmp.message = midievent->midiData[0] & 0xF0;
	tmp.channel = midievent->midiData[0] & 0x0F;
	tmp.data1   = midievent->midiData[1];
	tmp.data2   = midievent->midiData[2];

	return addMidiMsg(tmp);
}

int CMidiMsg::addMidiMsg(MidiMessage msg)
{
	// obt@ςȂ0Ԃ
	if (num >= MIDIMSG_MAXNUM)
		return 0;

	buf[num] = msg;

	num++;
	if (num > MIDIMSG_MAXNUM) {num = MIDIMSG_MAXNUM;}

	return 1;
};

MidiMessage CMidiMsg::getMidiMsg()
{
	MidiMessage tmp = buf[cur];

	num--;
	if( num < 0) { num = 0; }

	cur++;
	if( cur >= MIDIMSG_MAXNUM) { cur = MIDIMSG_MAXNUM -1;}

	return tmp;
}

VstInt32 CMidiMsg::getMidiMessageNum()
{
	return num;
}

VstInt32 CMidiMsg::getNextDeltaFrames()
{
	return buf[cur].deltaFrames;
}


void CMidiMsg::midiProc(MidiMessage mididata)
{
	short msb = 0;
	short lsb = 0;
	short pitchbend = 0;

	switch (mididata.message & 0xF0)
	{
	case 0x80: // m[gItbZ[W
		onMidiKeyOff(mididata.channel, mididata.data1, mididata.data2);
		break;

	case 0x90: // m[gIbZ[W
		// FVeB0̏ꍇ̓m[gItbZ[WƂĈ
		if (mididata.data2 > 0)
		{
			onMidiKeyOn(mididata.channel, mididata.data1, mididata.data2);
		}
		else
		{
			onMidiKeyOff(mididata.channel, mididata.data1, mididata.data2);
		}
		break;

	case 0xA0: // L[vbV[bZ[W
		onMidiPoliKeyPress(mididata.channel, mididata.data1, mididata.data2);
		break;

	case 0xB0: // Rg[`FWbZ[W
		onMidiControlChange(mididata.channel, mididata.data1, mididata.data2);
		break;

	case 0xC0: // vO`FWbZ[W
		onMidiProgramChange(mididata.channel, mididata.data1);
		break;

	case 0xD0: // `lvbV[bZ[W
		onMidiChannelPress(mididata.channel, mididata.data1);
		break;

	case 0xE0: // sb`xhbZ[W
		lsb = (mididata.data1);
		msb = (mididata.data2);
		pitchbend = ((msb << 7) | lsb) - 8192;
		onMidiPitchBend(mididata.channel, pitchbend);
		break;

	case 0xF0: // VXebZ[W
		onMidiSystemMessage((mididata.message & 0xF0) | (mididata.channel & 0x0F), mididata.data1, mididata.data2);
		break;
	}
}

void CMidiMsg::onMidiSystemMessage(unsigned char systemmsg, unsigned char data1, unsigned char data2)
{

	short lsb = (data1);
	short msb = (data2);
	switch(systemmsg)
	{
		case 0xF1: onMidiQuarterFrame(data1); break;
		case 0xF2: onMidiSongPointer( (msb << 7) | lsb );  break;
		case 0xF3: onMidiSongSelect(data1); break;
		case 0xF6: onMidiChainRequest();    break;
		case 0xF8: onMidiTimingClock();     break;
		case 0xFA: onMidiStart();           break;
		case 0xFB: onMidiContinue();        break;
		case 0xFC: onMidiStop();            break;
		case 0xFE: onMidiActiveSensing();   break;
		case 0xFF: onMidiSystemReset();     break;
		default:   break;
	}
}

void CMidiMsg::onMidiControlChange(unsigned char channel, unsigned char cc, unsigned char value)
{
	switch(cc)
	{
	case   0:  onMidiCC000 (channel, value); break;
	case   1:  onMidiCC001 (channel, value); break;
	case   2:  onMidiCC002 (channel, value); break;
	case   3:  onMidiCC003 (channel, value); break;
	case   4:  onMidiCC004 (channel, value); break;
	case   5:  onMidiCC005 (channel, value); break;
	case   6:  onMidiCC006 (channel, value); break;
	case   7:  onMidiCC007 (channel, value); break;
	case   8:  onMidiCC008 (channel, value); break;
	case   9:  onMidiCC009 (channel, value); break;
	case  10:  onMidiCC010 (channel, value); break;
	case  11:  onMidiCC011 (channel, value); break;
	case  12:  onMidiCC012 (channel, value); break;
	case  13:  onMidiCC013 (channel, value); break;
	case  14:  onMidiCC014 (channel, value); break;
	case  15:  onMidiCC015 (channel, value); break;
	case  16:  onMidiCC016 (channel, value); break;
	case  17:  onMidiCC017 (channel, value); break;
	case  18:  onMidiCC018 (channel, value); break;
	case  19:  onMidiCC019 (channel, value); break;
	case  20:  onMidiCC020 (channel, value); break;
	case  21:  onMidiCC021 (channel, value); break;
	case  22:  onMidiCC022 (channel, value); break;
	case  23:  onMidiCC023 (channel, value); break;
	case  24:  onMidiCC024 (channel, value); break;
	case  25:  onMidiCC025 (channel, value); break;
	case  26:  onMidiCC026 (channel, value); break;
	case  27:  onMidiCC027 (channel, value); break;
	case  28:  onMidiCC028 (channel, value); break;
	case  29:  onMidiCC029 (channel, value); break;
	case  30:  onMidiCC030 (channel, value); break;
	case  31:  onMidiCC031 (channel, value); break;
	case  32:  onMidiCC032 (channel, value); break;
	case  33:  onMidiCC033 (channel, value); break;
	case  34:  onMidiCC034 (channel, value); break;
	case  35:  onMidiCC035 (channel, value); break;
	case  36:  onMidiCC036 (channel, value); break;
	case  37:  onMidiCC037 (channel, value); break;
	case  38:  onMidiCC038 (channel, value); break;
	case  39:  onMidiCC039 (channel, value); break;
	case  40:  onMidiCC040 (channel, value); break;
	case  41:  onMidiCC041 (channel, value); break;
	case  42:  onMidiCC042 (channel, value); break;
	case  43:  onMidiCC043 (channel, value); break;
	case  44:  onMidiCC044 (channel, value); break;
	case  45:  onMidiCC045 (channel, value); break;
	case  46:  onMidiCC046 (channel, value); break;
	case  47:  onMidiCC047 (channel, value); break;
	case  48:  onMidiCC048 (channel, value); break;
	case  49:  onMidiCC049 (channel, value); break;
	case  50:  onMidiCC050 (channel, value); break;
	case  51:  onMidiCC051 (channel, value); break;
	case  52:  onMidiCC052 (channel, value); break;
	case  53:  onMidiCC053 (channel, value); break;
	case  54:  onMidiCC054 (channel, value); break;
	case  55:  onMidiCC055 (channel, value); break;
	case  56:  onMidiCC056 (channel, value); break;
	case  57:  onMidiCC057 (channel, value); break;
	case  58:  onMidiCC058 (channel, value); break;
	case  59:  onMidiCC059 (channel, value); break;
	case  60:  onMidiCC060 (channel, value); break;
	case  61:  onMidiCC061 (channel, value); break;
	case  62:  onMidiCC062 (channel, value); break;
	case  63:  onMidiCC063 (channel, value); break;
	case  64:  onMidiCC064 (channel, value); break;
	case  65:  onMidiCC065 (channel, value); break;
	case  66:  onMidiCC066 (channel, value); break;
	case  67:  onMidiCC067 (channel, value); break;
	case  68:  onMidiCC068 (channel, value); break;
	case  69:  onMidiCC069 (channel, value); break;
	case  70:  onMidiCC070 (channel, value); break;
	case  71:  onMidiCC071 (channel, value); break;
	case  72:  onMidiCC072 (channel, value); break;
	case  73:  onMidiCC073 (channel, value); break;
	case  74:  onMidiCC074 (channel, value); break;
	case  75:  onMidiCC075 (channel, value); break;
	case  76:  onMidiCC076 (channel, value); break;
	case  77:  onMidiCC077 (channel, value); break;
	case  78:  onMidiCC078 (channel, value); break;
	case  79:  onMidiCC079 (channel, value); break;
	case  80:  onMidiCC080 (channel, value); break;
	case  81:  onMidiCC081 (channel, value); break;
	case  82:  onMidiCC082 (channel, value); break;
	case  83:  onMidiCC083 (channel, value); break;
	case  84:  onMidiCC084 (channel, value); break;
	case  85:  onMidiCC085 (channel, value); break;
	case  86:  onMidiCC086 (channel, value); break;
	case  87:  onMidiCC087 (channel, value); break;
	case  88:  onMidiCC088 (channel, value); break;
	case  89:  onMidiCC089 (channel, value); break;
	case  90:  onMidiCC090 (channel, value); break;
	case  91:  onMidiCC091 (channel, value); break;
	case  92:  onMidiCC092 (channel, value); break;
	case  93:  onMidiCC093 (channel, value); break;
	case  94:  onMidiCC094 (channel, value); break;
	case  95:  onMidiCC095 (channel, value); break;
	case  96:  onMidiCC096 (channel, value); break;
	case  97:  onMidiCC097 (channel, value); break;
	case  98:  onMidiCC098 (channel, value); break;
	case  99:  onMidiCC099 (channel, value); break;
	case 100:  onMidiCC100 (channel, value); break;
	case 101:  onMidiCC101 (channel, value); break;
	case 102:  onMidiCC102 (channel, value); break;
	case 103:  onMidiCC103 (channel, value); break;
	case 104:  onMidiCC104 (channel, value); break;
	case 105:  onMidiCC105 (channel, value); break;
	case 106:  onMidiCC106 (channel, value); break;
	case 107:  onMidiCC107 (channel, value); break;
	case 108:  onMidiCC108 (channel, value); break;
	case 109:  onMidiCC109 (channel, value); break;
	case 110:  onMidiCC110 (channel, value); break;
	case 111:  onMidiCC111 (channel, value); break;
	case 112:  onMidiCC112 (channel, value); break;
	case 113:  onMidiCC113 (channel, value); break;
	case 114:  onMidiCC114 (channel, value); break;
	case 115:  onMidiCC115 (channel, value); break;
	case 116:  onMidiCC116 (channel, value); break;
	case 117:  onMidiCC117 (channel, value); break;
	case 118:  onMidiCC118 (channel, value); break;
	case 119:  onMidiCC119 (channel, value); break;
	case 120:  onMidiCC120 (channel, value); break;
	case 121:  onMidiCC121 (channel, value); break;
	case 122:  onMidiCC122 (channel, value); break;
	case 123:  onMidiCC123 (channel, value); break;
	case 124:  onMidiCC124 (channel, value); break;
	case 125:  onMidiCC125 (channel, value); break;
	case 126:  onMidiCC126 (channel, value); break;
	case 127:  onMidiCC127 (channel, value); break;
	default:   break;
	}
}